package cn.com.ty.lift.push.jiguang;

import cn.jiguang.common.resp.APIConnectionException;
import cn.jiguang.common.resp.APIRequestException;
import cn.jpush.api.push.PushResult;
import cn.jpush.api.push.model.Options;
import cn.jpush.api.push.model.Platform;
import cn.jpush.api.push.model.PushPayload;
import cn.jpush.api.push.model.audience.Audience;
import cn.jpush.api.push.model.notification.AndroidNotification;
import cn.jpush.api.push.model.notification.IosAlert;
import cn.jpush.api.push.model.notification.IosNotification;
import cn.jpush.api.push.model.notification.Notification;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collection;

@Slf4j
@Service
public class JiGuangServiceImpl implements JiGuangService {
    @Autowired
    private JpushConfig jpushConfig;
    private IosAlert buildTitleAndBody(String title,String content){
        return IosAlert.newBuilder().setTitleAndBody(title, null , content).build();
    }

    private IosAlert buildTitleAndBody(JiGuangMessage msg){
        return buildTitleAndBody(msg.getTitile(), msg.getContent());
    }
    @Override
    public boolean pushAll(JiGuangMessage msg) {
        return push(PushPayload.newBuilder()
                        .setPlatform(Platform.all())  //所有平台的用户
                        .setAudience(Audience.all())//registrationId指定用户
                        .setNotification(Notification.newBuilder()
                                .addPlatformNotification(IosNotification.newBuilder() //发送ios
                                        .setAlert(buildTitleAndBody(msg)) //消息体
                                        .setBadge(1)
                                        .setSound("happy") //ios提示音
                                        .addExtras(msg.getExtras()) //附加参数
                                        .build())
                                .addPlatformNotification(AndroidNotification.newBuilder() //发送android
                                        .addExtras(msg.getExtras()) //附加参数
                                        .setTitle(msg.getTitile())
                                        .setAlert(msg.getContent()) //消息体
                                        .build())
                                .build()
                        )
                        .setOptions(Options.newBuilder().setApnsProduction(jpushConfig.getApnsProduction()).build())//指定开发环境 true为生产模式 false 为测试模式 (android不区分模式,ios区分模式)
//                .setMessage(Message.newBuilder().setMsgContent(msg.getContent()).addExtras(msg.getExtras()).build())//自定义信息
                        .build()
        );

    }

    @Override
    public boolean pushIos(JiGuangMessage msg) {
        return push(PushPayload.newBuilder()
                        .setPlatform(Platform.ios())  //所有平台的用户
                        .setAudience(Audience.all())//registrationId指定用户
                        .setNotification(Notification.newBuilder()
                                .addPlatformNotification(IosNotification.newBuilder() //发送ios
                                        .setAlert(buildTitleAndBody(msg)) //消息体
                                        .setBadge(+1)
                                        .setSound("happy") //ios提示音
                                        .addExtras(msg.getExtras()) //附加参数
                                        .build())
                                .build()
                        )
                        .setOptions(Options.newBuilder().setApnsProduction(jpushConfig.getApnsProduction()).build())//指定开发环境 true为生产模式 false 为测试模式 (android不区分模式,ios区分模式)
//                .setMessage(Message.newBuilder().setMsgContent(msg.getContent()).addExtras(msg.getExtras()).build())//自定义信息
                        .build()
        );
    }

    @Override
    public boolean pushIos(JiGuangMessage msg, String... registids) {
        return push(PushPayload.newBuilder()
                        .setPlatform(Platform.ios())  //所有平台的用户
                        .setAudience(Audience.registrationId(registids))//registrationId指定用户
                        .setNotification(Notification.newBuilder()
                                .addPlatformNotification(IosNotification.newBuilder() //发送ios
                                        .setAlert(buildTitleAndBody(msg)) //消息体
                                        .setBadge(+1)
                                        .setSound("happy") //ios提示音
                                        .addExtras(msg.getExtras()) //附加参数
                                        .build())
                                .build()
                        )
                        .setOptions(Options.newBuilder().setApnsProduction(jpushConfig.getApnsProduction()).build())//指定开发环境 true为生产模式 false 为测试模式 (android不区分模式,ios区分模式)
//                .setMessage(Message.newBuilder().setMsgContent(msg.getContent()).addExtras(msg.getExtras()).build())//自定义信息
                        .build()
        );
    }

    @Override
    public boolean pushIos(JiGuangMessage msg, Collection registids) {
        return push(PushPayload.newBuilder()
                        .setPlatform(Platform.ios())  //所有平台的用户
                        .setAudience(Audience.registrationId(registids))//registrationId指定用户
                        .setNotification(Notification.newBuilder()
                                .addPlatformNotification(IosNotification.newBuilder() //发送ios
                                        .setAlert(buildTitleAndBody(msg)) //消息体
                                        .setBadge(+1)
                                        .setSound("happy") //ios提示音
                                        .addExtras(msg.getExtras()) //附加参数
                                        .build())
                                .build()
                        )
                        .setOptions(Options.newBuilder().setApnsProduction(jpushConfig.getApnsProduction()).build())//指定开发环境 true为生产模式 false 为测试模式 (android不区分模式,ios区分模式)
//                .setMessage(Message.newBuilder().setMsgContent(msg.getContent()).addExtras(msg.getExtras()).build())//自定义信息
                        .build()
        );
    }

    @Override
    public boolean pushAndroid(JiGuangMessage msg) {
        return push(PushPayload.newBuilder()
                        .setPlatform(Platform.android())  //所有平台的用户
                        .setAudience(Audience.all())//registrationId指定用户
                        .setNotification(Notification.newBuilder()
                                .addPlatformNotification(AndroidNotification.newBuilder() //发送android
                                        .addExtras(msg.getExtras()) //附加参数
                                        .setTitle(msg.getTitile())
                                        .setAlert(msg.getContent()) //消息体
                                        .build())
                                .build()
                        )
                        .setOptions(Options.newBuilder().setApnsProduction(true).build())//指定开发环境 true为生产模式 false 为测试模式 (android不区分模式,ios区分模式)
//                .setMessage(Message.newBuilder().setMsgContent(msg.getContent()).addExtras(msg.getExtras()).build())//自定义信息
                        .build()
        );
    }

    @Override
    public boolean pushAndroid(JiGuangMessage msg, String... registids) {
        return push(PushPayload.newBuilder()
                        .setPlatform(Platform.android())  //所有平台的用户
                        .setAudience(Audience.registrationId(registids))//registrationId指定用户
                        .setNotification(Notification.newBuilder()
                                .addPlatformNotification(AndroidNotification.newBuilder() //发送android
                                        .addExtras(msg.getExtras()) //附加参数
                                        .setTitle(msg.getTitile())
                                        .setAlert(msg.getContent()) //消息体
                                        .build())
                                .build()
                        )
                        .setOptions(Options.newBuilder().setApnsProduction(true).build())//指定开发环境 true为生产模式 false 为测试模式 (android不区分模式,ios区分模式)
//                .setMessage(Message.newBuilder().setMsgContent(msg.getContent()).addExtras(msg.getExtras()).build())//自定义信息
                        .build()
        );
    }

    @Override
    public boolean pushAndroid(JiGuangMessage msg, Collection registids) {
        return push(PushPayload.newBuilder()
                        .setPlatform(Platform.android())  //所有平台的用户
                        .setAudience(Audience.registrationId(registids))//registrationId指定用户
                        .setNotification(Notification.newBuilder()
                                .addPlatformNotification(AndroidNotification.newBuilder() //发送android
                                        .addExtras(msg.getExtras()) //附加参数
                                        .setTitle(msg.getTitile())
                                        .setAlert(msg.getContent()) //消息体
                                        .build())
                                .build()
                        )
                        .setOptions(Options.newBuilder().setApnsProduction(true).build())//指定开发环境 true为生产模式 false 为测试模式 (android不区分模式,ios区分模式)
//                .setMessage(Message.newBuilder().setMsgContent(msg.getContent()).addExtras(msg.getExtras()).build())//自定义信息
                        .build()
        );
    }

    @Override
    public boolean push(PushPayload pushPayload) {
        log.info("发送极光推送请求: {}", pushPayload);
        PushResult result = null;
        try {
            result = jpushConfig.getJPushClient().sendPush(pushPayload);
        } catch (APIConnectionException e) {
            log.error("极光推送连接异常: ", e);
        } catch (APIRequestException e) {
            log.error("极光推送请求异常: ", e);
        }
        if (result != null && result.isResultOK()) {
            log.info("极光推送请求成功: {}", result);
            return true;
        } else {
            log.info("极光推送请求失败: {}", result);
            return false;
        }
    }
}
