package cn.com.ty.lift.ud.chat.controller.query;

import java.util.Date;

import cn.com.ty.lift.ud.common.BasePageQuery;
import org.apache.commons.lang3.StringUtils;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import cn.com.ty.lift.ud.chat.mapper.entity.ChatMsgEntity;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

/**
 * 聊天记录 查询条件
 * 
 * @author Xson
 * @date 2020-04-22
 */
@Data
@ApiModel(description = "聊天记录 查询条件")
public final class ChatMsgQuery extends BasePageQuery {

    /**
     * 主键
     */
    @ApiModelProperty("主键")
    private Integer id;

    /**
     * session 会话
     */
    @ApiModelProperty("session 会话")
    private String sessionid;

    /**
     * 来自
     */
    @ApiModelProperty("来自")
    private Long fromUser;

    /**
     * 到达
     */
    @ApiModelProperty("到达")
    private Long toUser;

    /**
     * 信息 聊天信息
     */
    @ApiModelProperty("信息 聊天信息")
    private String msg;

    /**
     * 状态 0:正常1:屏蔽
     */
    @ApiModelProperty("状态 1:正常0:屏蔽")
    private Integer statuz;

    /**
     * 创建人
     */
    @ApiModelProperty("创建人")
    private String createBy;

    /**
     * 创建时间
     */
    @ApiModelProperty("创建时间")
    private Date createTime;

    /**
     * 更新人
     */
    @ApiModelProperty("更新人")
    private String updateBy;

    /**
     * 更新时间
     */
    @ApiModelProperty("更新时间")
    private Date updateTime;

    @ApiModelProperty("起始时间")
    private Date starDate;

    @ApiModelProperty("结束时间")
    private Date endDate;

    @ApiModelProperty("用户昵称")
    private String userName;

    /**
     * 查询条件
     * 
     * @return
     */
    @Override
    public Wrapper condition() {
        QueryWrapper<ChatMsgEntity> query = new QueryWrapper<>();
        if (StringUtils.isNotBlank(sessionid)) {
            query.eq("t.sessionid", sessionid);
        }

        if (null != statuz) {
            query.eq("t.statuz", statuz);
        }

        if (StringUtils.isNotBlank(msg)) {
            query.like("t.msg", msg);
        }

        if (null != starDate && null != endDate) {
            query.between("t.create_time", starDate, endDate);
        }

        if (StringUtils.isNotBlank(userName)) {
            query.like("users.name", userName);
        }

        if (null != getSort()) {
            if (getSort().getOrder().indexOf("asc") != -1 || getSort().getOrder().indexOf("ASC") != -1) {
                query.orderByAsc("t." + getSort().getOrderBy());
            } else {
                query.orderByDesc("t." + getSort().getOrderBy());
            }
        }
        query.eq("1", 1);
        return query;
    }

    /**
     * 分页条件
     * 
     * @return
     */
    @Override
    public IPage page() {
        Page<ChatMsgEntity> page = new Page<>(getPageNum(), getPageSize());
        return page;
    }
}
