package cn.com.ty.lift.ud.chat.controller.query;

import cn.com.ty.lift.ud.common.BasePageQuery;
import cn.com.ty.lift.ud.chat.mapper.entity.ChatSessionEntity;

import java.util.Date;

import org.apache.commons.lang3.StringUtils;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

/**
 * 会话 查询条件
 * 
 * @author Xson
 * @date 2020-04-22
 */
@Data
@ApiModel(description = "会话 查询条件")
public final class ChatSessionQuery extends BasePageQuery {

    /**
     * 主键
     */
    @ApiModelProperty("主键")
    private Integer id;

    /**
     * 会话
     */
    @ApiModelProperty("会话")
    private String sessionid;

    /**
     * 数据id
     */
    @ApiModelProperty("数据id")
    private Integer dataId;

    /**
     * 类型 1:问诊2:问诊3:私聊
     */
    @ApiModelProperty("类型 1:问诊2:问诊3:私聊")
    private Integer dataTable;

    /**
     * 用户 会话成员
     */
    @ApiModelProperty("用户 会话成员")
    private String userId;

    /**
     * 发起人 发起人
     */
    @ApiModelProperty("发起人 发起人")
    private String createUserId;

    /**
     * 状态 0:正常1:屏蔽
     */
    @ApiModelProperty("状态 1:正常0:屏蔽")
    private Integer statuz;

    /**
     * 创建人
     */
    @ApiModelProperty("创建人")
    private String createBy;

    /**
     * 创建时间
     */
    @ApiModelProperty("创建时间")
    private Date createTime;

    /**
     * 更新人
     */
    @ApiModelProperty("更新人")
    private String updateBy;

    /**
     * 更新时间
     */
    @ApiModelProperty("更新时间")
    private Date updateTime;

    @ApiModelProperty("起始时间")
    private Date starDate;

    @ApiModelProperty("结束时间")
    private Date endDate;
    
    @ApiModelProperty("发起人")
    private String userName;

    @ApiModelProperty("诊单描述")
    private String caseExpression;

    /**
     * 查询条件
     * 
     * @return
     */
    @Override
    public Wrapper condition() {
        QueryWrapper<ChatSessionEntity> query = new QueryWrapper<>();

        if (StringUtils.isNotBlank(sessionid)) {
            query.eq("t.sessionid", sessionid);
        }

        if (null != statuz) {
            query.eq("t.statuz", statuz);
        }

        if (null != dataTable) {
            query.eq("t.data_table", dataTable);
        }
        if (null != userId) {
            query.eq("t.user_id", userId);
        }

        if (null != starDate && null != endDate) {
            query.between("t.create_time", starDate, endDate);
        }
        if (StringUtils.isNotBlank(userName)) {
            query.like("users.name",userName);
        }
        if (StringUtils.isNotBlank(caseExpression)) {
            query.like("caseExpression",caseExpression);
        }
        //按修改时间倒序
//        query.orderByDesc("t.update_time");
        query.isNotNull("t.create_user_id");
        if (null != getSort()) {
            if (getSort().getOrder().indexOf("asc") != -1 || getSort().getOrder().indexOf("ASC") != -1) {
                query.orderByAsc(getSort().getOrderBy());
            } else {
                query.orderByDesc(getSort().getOrderBy());
            }
        }
        query.eq("1", 1);
        return query;
    }

    /**
     * 分页条件
     * 
     * @return
     */
    @Override
    public IPage page() {
        Page<ChatSessionEntity> page = new Page<>(getPageNum(), getPageSize());
        return page;
    }

}
