package cn.com.ty.lift.ud.expertComment.controller.query;

import cn.com.ty.lift.ud.common.BasePageQuery;
import cn.com.ty.lift.ud.expertComment.mapper.entity.ExpertCommentEntity;

import java.util.Date;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

/**
 * 诊单评论  查询条件
 * 
 * @author Xson
 * @date 2020-05-06
 */
@Data
@ApiModel(description = "诊单评论  查询条件")
public final class ExpertCommentQuery extends BasePageQuery {

    /**
     * 主键
     */
    @ApiModelProperty("主键")
     private Integer id;

    /**
     * 创建人
     */
    @ApiModelProperty("创建人")
     private String createBy;

    /**
     * 创建时间
     */
    @ApiModelProperty("创建时间")
     private Date createTime;

    /**
     * 更新人
     */
    @ApiModelProperty("更新人")
     private String updateBy;

    /**
     * 更新时间
     */
    @ApiModelProperty("更新时间")
     private Date updateTime;

    /**
     * 技术服务
     */
    @ApiModelProperty("技术服务")
    private Integer tech;

    /**
     * 反应速度
     */
    @ApiModelProperty("反应速度")
     private Integer responseSpeed;

    /**
     * 服务态度
     */
    @ApiModelProperty("服务态度")
    private Integer attitude;

    /**
     * 评论内容
     */
    @ApiModelProperty("评论内容")
    private String comment;

    /**
     * 专家id
     */
    @ApiModelProperty("专家id")
     private Long expertId;

    /**
     * 评论人id
     */
    @ApiModelProperty("评论人id")
    private Long userId;


    /**
     * 状态 1: 屏蔽 0:不屏蔽
     */
    @ApiModelProperty("状态 1: 屏蔽 0:不屏蔽")
    private Integer statuz;

    /**
     * 专家姓名，后面加的
     */
    @ApiModelProperty("专家姓名")
    private String expertName;

    /**
     * 评论人姓名，后面加的
     */
    @ApiModelProperty("评论人姓名")
    private String userName;

    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @ApiModelProperty("起始时间")
    private Date starDate;

    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @ApiModelProperty("结束时间")
    private Date endDate;

    /**
     * 诊单ID
     */
    @ApiModelProperty("诊单ID")
    private Long lifeCaseId;

    /**
     * 查询条件
     * @return
     */
    @Override
    public Wrapper condition() {
        QueryWrapper<ExpertCommentEntity> query = new QueryWrapper<>();
        //专家
        if (StringUtils.isNotBlank(expertName)) {
            query.like("info.name", expertName);
        }
        //评论人
        if (StringUtils.isNotBlank(userName)) {
            query.like("us.name", userName);
        }
        //专家ID
        if( null !=expertId ){
            query.eq("ec.expert_id",expertId);
        }
        //状态
        if( null !=statuz ){
            query.eq("ec.statuz",statuz);
        }
        //评论时间
        if(null != starDate && null != endDate){
            query.between("ec.create_time", starDate, endDate);
        }
        //按时间倒序排序
        query.orderByDesc("ec.create_time");
        query.eq("1",1);
        return query;
    }

    /**
     * 分页条件
     * @return
     */
    @Override
    public IPage page() {
        Page<ExpertCommentEntity> page = new Page<>(getPageNum(), getPageSize());
        return page;
    }

}

