package cn.com.ty.lift.ud.liftCaseAppeal.controller.query;

import cn.com.ty.lift.ud.common.BasePageQuery;
import cn.com.ty.lift.ud.liftCaseAppeal.mapper.entity.LiftCaseAppealEntity;

import java.util.Date;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

/**
 * 诊单申诉 查询条件
 * 
 * @author Xson
 * @date 2020-04-23
 */
@Data
@ApiModel(description = "诊单申诉 查询条件")
public final class LiftCaseAppealQuery extends BasePageQuery {

    /**
     * 主键
     */
    @ApiModelProperty("主键")
     private Integer id;

    /**
     * 创建人
     */
    @ApiModelProperty("创建人")
     private String createBy;

    /**
     * 创建时间
     */
    @ApiModelProperty("创建时间")
     private Date createTime;

    /**
     * 更新人
     */
    @ApiModelProperty("更新人")
     private String updateBy;

    /**
     * 更新时间
     */
    @ApiModelProperty("更新时间")
     private Date updateTime;

    /**
     * 申诉内容
     */
    @ApiModelProperty("申诉内容")
    private String content;

    /**
     * 图片
     */
    @ApiModelProperty("图片")
    private String imgs;

    /**
     * 状态 1未结束  2 驳回 3 已结束 4已退款
     */
    @ApiModelProperty("状态 1未结束  2 驳回 3 已结束 4已退款")
    private Integer statuz;

    /**
     * 数据id
     */
    @ApiModelProperty("数据id")
     private Long dataId;

    /**
     * 申述人ID
     */
    @ApiModelProperty("申述人")
     private String appealerId;

    /**
     * 取消原因
     */
    @ApiModelProperty("取消原因")
     private String cancelSeason;


    /**
     * 申诉人姓名，后面加的
     */
    @ApiModelProperty("申诉人姓名")
    private String userName;


    /**
     * 专家姓名
     */
    @ApiModelProperty("专家姓名")
    private String expertName;


    /**
     * 查询条件
     * @return
     */
    @Override
    public Wrapper condition() {
        QueryWrapper<LiftCaseAppealEntity> query = new QueryWrapper<>();
        if (null != appealerId) {
            query.eq("a.appealer_id", appealerId);
        }
        //申诉人
        if (StringUtils.isNotBlank(userName)) {
            query.like("users.name", userName);
        }
        //专家
        if (StringUtils.isNotBlank(expertName)) {
            query.like("expert.name", expertName);
        }
        //状态
        if (null != statuz) {
            query.eq("a.statuz", statuz);
        }
        //显示顺序：时间倒序，未结束，驳回，已结束
        query.orderByDesc("a.create_time").orderByAsc("a.statuz");
        query.eq("1",1);
        return query;
    }

    /**
     * 分页条件
     * @return
     */
    @Override
    public IPage page() {
        Page<LiftCaseAppealEntity> page = new Page<>(getPageNum(), getPageSize());
        return page;
    }

}

