package cn.com.ty.lift.ud.maintenanceCompany.mapper.entity;

import cn.com.ty.lift.ud.common.BaseEntity;
import cn.com.ty.lift.ud.common.LocalDateTimeConverter;
import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.Date;

/**
 * 维保公司
 * 
 * @author Xson
 * @date 2020-05-19
 */
@Data
@ApiModel(description = "维保公司")
@TableName(value = "maintenance_company")
public class MaintenanceCompanyEntity extends BaseEntity {

    /**
     * 公司名称
     */
    @ApiModelProperty("公司名称")
    private String name;

    /**
     * 公司地址
     */
    @ApiModelProperty("公司地址")
    private String address;

    /**
     * 注册资金
     */
    @ApiModelProperty("注册资金")
    private BigDecimal registeredCapital;

    /**
     * 企业法人
     */
    @ApiModelProperty("企业法人")
    private String corporator;

    /**
     * 联系电话
     */
    @ApiModelProperty("联系电话")
    private String telephone;

    /**
     * 主要业务
     */
    @ApiModelProperty("主要业务")
    private String mainbusiness;

    /**
     * 营业执照编号
     */
    @ApiModelProperty("营业执照编号")
    private String businessLicenseCode;

    private String businessLicenseAnnex;

    /**
     * 备注
     */
    @ApiModelProperty("备注")
    private String remarks;

    /**
     * 是否认证 '0 未认证 1 已认证 2 审核中 3 认证失败'
     */
    @ApiModelProperty("是否认证  '0 未认证 1 已认证 2 审核中 3 认证失败'")
    private Integer isCertificated;

    /**
     * 状态 '1：生效 2：系统创建'
     */
    @ApiModelProperty("状态  '1：生效 2：系统创建'")
    private Integer status;

    private Integer limitedNum;

    /**
     * 服务总积分
     */
    @ApiModelProperty("服务总积分")
    private Integer serviceTotal;

    /**
     * 满意度总积分
     */
    @ApiModelProperty("满意度总积分")
    private Integer satisfactionTotal;

    private Integer serviceCount;

    /**
     * 用户等级
     */
    @ApiModelProperty("用户等级")
    private Integer level;

    /**
     * 团队图片
     */
    @ApiModelProperty("团队图片")
    private String logoImg;

    /**
     * 维保积分
     */
    @ApiModelProperty("维保积分")
    private Integer maintenanceTotal;

    /**
     * 急修积分
     */
    @ApiModelProperty("急修积分")
    private Integer emergencyTotal;

    /**
     * 创建电梯积分
     */
    @ApiModelProperty("创建电梯积分")
    private Integer liftTotal;

    /**
     * 是否开启打卡
     */
    @ApiModelProperty("是否开启打卡")
    private Integer isClock;

    /**
     * 上午上班时间
     */
    @ApiModelProperty("上午上班时间")
    private Date topTime;

    /**
     * 上午下班时间
     */
    @ApiModelProperty("上午下班时间")
    private Date downTime;

    /**
     * 是否开启上下班打卡
     */
    @ApiModelProperty("是否开启上下班打卡")
    private Integer isUpdown;

    /**
     * 是否vip
     */
    @ApiModelProperty("是否vip")
    private Integer vipFlag;

    /**
     * 下午上班时间
     */
    @ApiModelProperty("下午上班时间")
    private Date topTimePm;

    /**
     * 下午下班时间
     */
    @ApiModelProperty("下午下班时间")
    private Date downTimePm;

    /**
     * 加班上班时间
     */
    @ApiModelProperty("加班上班时间")
    private Date topTimeOt;

    /**
     * 加班下班时间
     */
    @ApiModelProperty("加班下班时间")
    private Date downTimeOt;

    /**
     * 省份编码
     */
    @ApiModelProperty("省份编码")
    private String provinceCode;
    
    /**
     * 企业类型 1:民营企业2:国企3:外资
     */
    @ApiModelProperty("企业类型 1:民营企业2:国企3:外资")
    private Integer type;
    
    @ApiModelProperty("公司规模")
    private String companySize;
    
    /**
     * 创建时间 默认为系统当前时间
     */
    @ApiModelProperty(value = "创建时间")
    @TableField(value = "create_date", fill = FieldFill.INSERT)
    @JsonSerialize(using = LocalDateTimeConverter.class)
    private LocalDateTime createTime;

    /**
     * 创建者id
     */
    @ApiModelProperty(value = "创建人id")
    @TableField(value = "creator_id", fill = FieldFill.INSERT)
    private String createBy;

    /**
     * 修改时间 默认为系统当前时间
     */
    @ApiModelProperty(value = "修改时间")
    @TableField(value = "update_time", fill = FieldFill.INSERT_UPDATE)
    @JsonSerialize(using = LocalDateTimeConverter.class)
    private LocalDateTime updateTime;

    /**
     * 修改者id
     */
    @ApiModelProperty(value = "修改人")
    @TableField(value = "update_user_id", fill = FieldFill.INSERT_UPDATE)
    private String updateBy;

}
