package cn.com.ty.lift.ud.neety.task;

import java.time.ZoneOffset;
import java.util.*;
import java.util.stream.Collectors;

import cn.com.ty.lift.ud.chat.mapper.entity.ChatSessionEntity;
import cn.com.ty.lift.ud.chat.service.IChatSessionService;
import cn.com.ty.lift.ud.chat.service.impl.ChatSessionServiceImpl;
import cn.com.ty.lift.ud.redis.RedisUtils;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.context.annotation.DependsOn;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Component;

import cn.com.ty.lift.ud.utils.ApplicationContextUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 更新缓存中用户在线数据状态
 *
 * @author sKing
 * @Date 2020年6月8日
 */
@Slf4j
@Component
public class RedisRunner implements CommandLineRunner {
    @Autowired
    private RedisUtils redis;

    @Autowired
    private IChatSessionService sessionService;

    @Override
    public void run(String... args) throws Exception {
        log.info("##--------------------->room loading!");
        redis.deletePattern("ROOM:*");

        //1.先查询出当前用户所有有效房间，然后把房间用户信息放入到redis中
        List<ChatSessionEntity> sessionList = Optional.ofNullable(sessionService.findAllSession()).orElse(new ArrayList<>());
        Map<String, List<ChatSessionEntity>> groupbySessionid = sessionList.stream().collect(Collectors.groupingBy(ChatSessionEntity::getSessionid));
        groupbySessionid.keySet().forEach(sessionid -> {
            List<ChatSessionEntity> userList = groupbySessionid.get(sessionid);
            ChatSessionEntity fistUserinfo = userList.get(0);
            RoomInfo roomInfo = new RoomInfo();

            roomInfo.setSessionid(fistUserinfo.getSessionid());
            roomInfo.setDataTable(fistUserinfo.getDataTable());
            roomInfo.setCaseType(fistUserinfo.getCaseType());
            roomInfo.setAcceptStatus(fistUserinfo.getAcceptStatus());
            roomInfo.setExpression(fistUserinfo.getExpression());
            if(null == fistUserinfo.getCreateTime()) {
                roomInfo.setLastTime(null);
            }else {
                roomInfo.setLastTime(fistUserinfo.getCreateTime().toInstant(ZoneOffset.of("+8")).toEpochMilli());
            }

            List<UserInfo> users = new ArrayList<>();
            for (ChatSessionEntity chatSessionEntity : userList) {
                UserInfo user = new UserInfo();
                user.setUserId(chatSessionEntity.getTargetId());
                user.setAvatarUrl(chatSessionEntity.getAvatarUrl());
                user.setUserName(chatSessionEntity.getUserName());
                user.setIsRed(0);
                users.add(user);
            }
            roomInfo.setUserList(users);
            redis.set("ROOM:" + sessionid, JSONObject.toJSONString(roomInfo));
        });
        log.info("##--------------------->room loaded!");
    }

}
