package cn.com.ty.lift.ud.neety.task;

import cn.com.ty.lift.ud.chat.mapper.entity.ChatMsgEntity;
import cn.com.ty.lift.ud.chat.service.IChatMsgService;
import cn.com.ty.lift.ud.chat.service.IChatUnreadMsgService;
import cn.com.ty.lift.ud.neety.protocol.IMMessage;
import cn.com.ty.lift.ud.redis.RedisUtils;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.ZoneOffset;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;

/**
 * 同步缓存数据到mysql
 *
 * @author sKing
 */
@Slf4j
@Component
public class RedisTask {
    @Autowired
    private RedisUtils redis;
    @Autowired
    private IChatUnreadMsgService chatUnreadMsgService;
    @Autowired
    private IChatMsgService msgService;

    public RedisTask() {
        log.info("RedisTask init");
    }

    /**
     * 在凌晨时，讲聊天记录同步到数据库中，redis只放当天的数据
     */
    @Scheduled(cron = "0 0 1 * * ?")
    public void synRedisChatToDB() {
        log.info("执行保存未读消息操作！");
        Set<String> keys = redis.keys("ALLCHATS:*");
        for (String key : keys) {
            List<ChatMsgEntity> chatList = new ArrayList<>();
            List<String> list = redis.lRange(key, 0, -1);
            list.forEach(msg -> {
                ChatMsgEntity entity = new ChatMsgEntity();
                IMMessage imMessage = JSONObject.parseObject(msg, IMMessage.class);
                entity.setMsg(imMessage.getMsg());
                entity.setFromUser(imMessage.getFromUser());
                entity.setToUser(imMessage.getReceiver());
                entity.setType(imMessage.getType());
                entity.setSessionid(imMessage.getSessionid());
                entity.setStatuz(1);
                entity.setCreateBy(imMessage.getName());
                entity.setCreateTime(new Date(imMessage.getTime()).toInstant().atOffset(ZoneOffset.of("+8")).toLocalDateTime());
                chatList.add(entity);
            });

            if (!chatList.isEmpty()) {
                msgService.saveBatch(chatList);
            }
            redis.delete(key);
        }
        log.info("End. 保存未读消息操作结束！");

    }
}
