package cn.com.ty.lift.ud.payment.controller.query;

import cn.com.ty.lift.ud.payment.mapper.entity.TyBillEntity;

import java.math.BigDecimal;
import java.util.Date;

import org.apache.commons.lang3.StringUtils;

import cn.com.ty.lift.ud.common.BasePageQuery;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

/**
 * 天域账单流水 查询条件
 * 
 * @author Xson
 * @date 2020-04-23
 */
@Data
@ApiModel(description = "天域账单流水 查询条件")
public final class TyBillQuery extends BasePageQuery {

    /**
     * 主键
     */
    @ApiModelProperty("主键")
    private Integer id;

    /**
     * 交易额
     */
    @ApiModelProperty("交易额")
    private BigDecimal amount;

    /**
     * 账户余额
     */
    @ApiModelProperty("账户余额")
    private BigDecimal balance;

    /**
     * 交易类型 1：支出、2：收入
     */
    @ApiModelProperty("交易类型 1：支出、2：收入")
    private Integer type;

    /**
     * 支付渠道
     */
    @ApiModelProperty("支付渠道")
    private Integer payType;

    /**
     * 业务类型
     */
    @ApiModelProperty("业务类型")
    private Integer businessType;

    /**
     * 流水号
     */
    @ApiModelProperty("流水号")
    private String flowNum;

    /**
     * 备注
     */
    @ApiModelProperty("备注")
    private String descr;

    /**
     * 创建时间
     */
    @ApiModelProperty("创建时间")
    private Date starDate;


    /**
     * 更新时间
     */
    @ApiModelProperty("更新时间")
    private Date endDate;

    /**
     * 查询条件
     * 
     * @return
     */
    @Override
    public Wrapper condition() {
        QueryWrapper<TyBillEntity> query = new QueryWrapper<>();
        if (StringUtils.isNotBlank(flowNum)) {
            query.like("flow_num", flowNum);
        }
        if (null != type) {
            query.eq("type", type);
        }
        if (null != payType) {
            query.eq("pay_type", payType);
        }
        if (null != businessType) {
            query.eq("business_type", businessType);
        }
        if(null != starDate && null != endDate){
            query.between("create_time", starDate, endDate);
        }
        query.eq("1", 1);
        //按创建时间倒序
        query.orderByDesc("create_time");
        return query;
    }

    /**
     * 分页条件
     * 
     * @return
     */
    @Override
    public IPage page() {
        Page<TyBillEntity> page = new Page<>(getPageNum(), getPageSize());
        return page;
    }

}
