package cn.com.ty.lift.ud.payment.controller.query;

import cn.com.ty.lift.ud.payment.mapper.entity.UserWithdrawEntity;

import java.math.BigDecimal;
import java.util.Date;

import org.apache.commons.lang3.StringUtils;

import cn.com.ty.lift.ud.common.BasePageQuery;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

/**
 * 用户提现申请 查询条件
 * 
 * @author Xson
 * @date 2020-04-23
 */
@Data
@ApiModel(description = "用户提现申请 查询条件")
public final class UserWithdrawQuery extends BasePageQuery {

    /**
     * 主键
     */
    @ApiModelProperty("主键")
    private Integer id;

    /**
     * 用户
     */
    @ApiModelProperty("用户")
    private Integer userId;

    /**
     * 提现金额
     */
    @ApiModelProperty("提现金额")
    private BigDecimal amount;

    /**
     * 支付宝微信 1:微信2:支付宝
     */
    @ApiModelProperty("支付宝微信 1:微信2:支付宝")
    private Integer type;

    /**
     * 账号
     */
    @ApiModelProperty("账号")
    private String acount;

    /**
     * 名称
     */
    @ApiModelProperty("名称")
    private String name;

    /**
     * 状态 0:申请1:审核2:提现3:到账
     */
    @ApiModelProperty("状态 0:申请1:审核通过2:提现3:到账4、审核不通过',")
    private Integer statuz;

    /**
     * 创建人
     */
    @ApiModelProperty("创建人")
    private String createBy;

    /**
     * 创建时间
     */
    @ApiModelProperty("创建时间")
    private Date createTime;

    /**
     * 更新人
     */
    @ApiModelProperty("更新人")
    private String updateBy;

    /**
     * 更新时间
     */
    @ApiModelProperty("更新时间")
    private Date updateTime;

    /**
     * 用户
     */
    @ApiModelProperty("用户昵称")
    private String userName;
    
    @ApiModelProperty("手机号码")
    private String mobile;
    
    @ApiModelProperty("用户角色：1：是专家，0：不是专家")
    private Integer expertFlag;
    
    @ApiModelProperty("提现审核描述")
    private String mome;
    

    /**
     * 查询条件
     * 
     * @return
     */
    @Override
    public Wrapper condition() {
        QueryWrapper<UserWithdrawEntity> query = new QueryWrapper<>();
        if (StringUtils.isNotBlank(name)) {
            query.like("t.name", name);
        }
        if (StringUtils.isNotBlank(mobile)) {
            query.like("users.mobile", mobile);
        }
        if (StringUtils.isNotBlank(userName)) {
            query.like("users.name", userName);
        }
        if (null != statuz) {
            query.eq("t.statuz", statuz);
        }
        if (null != expertFlag) {
            query.eq("acc.expert_flag", expertFlag);
        }

        if (null != getSort()) {
            if (getSort().getOrder().indexOf("asc") != -1 || getSort().getOrder().indexOf("ASC") != -1) {
                query.orderByAsc("t." + getSort().getOrderBy());
            } else {
                query.orderByDesc("t." + getSort().getOrderBy());
            }
        }

        query.eq("1", 1);
        return query;
    }

    /**
     * 分页条件
     * 
     * @return
     */
    @Override
    public IPage page() {
        Page<UserWithdrawEntity> page = new Page<>(getPageNum(), getPageSize());
        return page;
    }

}
