package cn.com.ty.lift.ud.systemMenu.controller.query;

import cn.com.ty.lift.ud.common.BasePageQuery;
import cn.com.ty.lift.ud.systemMenu.mapper.entity.Menu;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

/**
 * 菜单表查询条件
 *
 * @author Xson
 * @date 2020-06-30
 */
@Data
@ApiModel(description = "查询条件")
public final class MenuQuery extends BasePageQuery {

    /**
     * 主键id
     */
    @ApiModelProperty("主键id")
    private Long id;

    /**
     * 父id
     */
    @ApiModelProperty("父id")
    private Long parentId;

    /**
     * 菜单名称
     */
    @ApiModelProperty("菜单名称")
    private String name;

    /**
     * 菜单编码
     */
    @ApiModelProperty("菜单编码")
    private String code;

    /**
     * 菜单路径
     */
    @ApiModelProperty("菜单路径")
    private String url;

    /**
     * 菜单类型：1：菜单名，2：操作
     */
    @ApiModelProperty("菜单类型：1：菜单名，2：操作")
    private String type;

    /**
     * 描述
     */
    @ApiModelProperty("描述")
    private String description;

    /**
     * 是否默认菜单，0表示不是，1表示是
     */
    @ApiModelProperty("是否默认菜单，0表示不是，1表示是")
    private Integer defaultFlag;


    /**
     * 查询条件
     *
     * @return
     */
    @Override
    public Wrapper condition() {
        QueryWrapper<MenuQuery> query = new QueryWrapper<>();
        query.eq(null != parentId, "parent_id", parentId)
                .eq(StringUtils.isNotBlank(type), "type", type)
                .eq(null != id, "id", id)
                .like(StringUtils.isNotBlank(name), "name", name);

        return query;
    }

    /**
     * 分页条件
     *
     * @return
     */
    @Override
    public IPage page() {
        Page<Menu> page = new Page<>(getPageNum(), getPageSize());
        return page;
    }

}

