package cn.com.ty.lift.ud.task;

import javax.annotation.Resource;

import cn.com.ty.lift.ud.chat.mapper.entity.LiftCaseEntity;
import cn.com.ty.lift.ud.userOrders.service.impl.CustomMultiThreadingService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import cn.com.ty.lift.ud.chat.service.ILiftCaseService;

import java.util.List;

/**
 * 专家订单超时，拒绝接单
 *
 * @author sKing
 */

@Component
public class RefusedLiftCaseTask {

    private static final Logger logger = LoggerFactory.getLogger(SureLiftCaseTask.class);

    @Resource(type = ILiftCaseService.class)
    private ILiftCaseService liftCaseService;

    private @Autowired
    CustomMultiThreadingService customMultiThreadingService;

    private static final int TIME = 1800;

    /**
     * 每隔5分钟，定时专家未接单，超时之后，拒绝
     * 线上必须用public ，本地用private也可以
     */
    @Scheduled(cron = "0 */5 * * * ?")
    public void refusedLiftCaseTask() {
        logger.info("20分钟之内不接单，自动拒绝！----------------开始");
        liftCaseService.refusedLiftCaseTask();
        logger.info("20分钟之内不接单，自动拒绝！----------------结束");
    }

    /**
     * 每隔3分钟，更新未接单颜色等级
     * 线上必须用public ，本地用private也可以
     */
    @Scheduled(cron = "0 */3 * * * ?")
    public void updateColorGrade() {
        logger.info("诊单提示，3分钟加一个级别----------------开始");
        liftCaseService.updateColorGrade();
        logger.info("诊单提示，3分钟加一个级别----------------开始");
    }


    /**
     * 每隔1分钟，查询可取消de 待支付的诊单
     */
    @Scheduled(cron = "0 */1 * * * ?")
    public void cancelLiftCase() {
        logger.info("自动取消待支付的诊单----------------开始");

        List<Long> liftCaseEntityList = liftCaseService.findListByCreateTimeAndStatuz(TIME, 0);

        if (liftCaseEntityList.size() > 0) {
            liftCaseEntityList.forEach(item -> customMultiThreadingService.liftCaseAysncTask(item));
        }

        logger.info("自动取消待支付的诊单----------------结束，共执行任务数：{}", liftCaseEntityList.size());
    }
}
