package cn.com.ty.lift.ud.templateBase.controller.query;

import cn.com.ty.lift.ud.common.BasePageQuery;
import cn.com.ty.lift.ud.question.mapper.entity.QuestionBankEntity;
import cn.com.ty.lift.ud.templateBase.mapper.entity.MaintenanceOptionEntity;

import java.util.Date;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

/**
 * 维保项 查询条件
 * 
 * @author Xson
 * @date 2020-04-23
 */
@Data
@ApiModel(description = "维保项 查询条件")
public final class MaintenanceOptionQuery extends BasePageQuery {

    /**
     * 主键
     */
    @ApiModelProperty("主键")
     private Integer id;

    /**
     * 维保项 维保项
     */
    @ApiModelProperty("维保项 维保项")
    private String opt;

    /**
     * 内容 维保项内容
     */
    @ApiModelProperty("内容 维保项内容")
    private String requirement;

    /**
     * 状态 0:上架1:下架
     */
    @ApiModelProperty("状态 0:上架1:下架")
    private Integer statuz;

    /**
     * 电梯类别(1:曳引梯;2:液压梯;3:杂物梯;4:自动扶梯;5:自动人行道)
     */
    @ApiModelProperty("电梯类别(1:曳引梯;2:液压梯;3:杂物梯;4:自动扶梯;5:自动人行道)")
     private String driveType;

    /**
     * 保养时间类型  ''1.半月，2.季度，3.半年，4.全年
     */
    @ApiModelProperty("保养时间类型  ''1.半月，2.季度，3.半年，4.全年")
     private String validTime;

    /**
     * 创建人
     */
    @ApiModelProperty("创建人")
     private String createBy;

    /**
     * 创建时间
     */
    @ApiModelProperty("创建时间")
     private Date createTime;

    /**
     * 更新人
     */
    @ApiModelProperty("更新人")
     private String updateBy;

    /**
     * 更新时间
     */
    @ApiModelProperty("更新时间")
     private Date updateTime;




    /**
     * 查询条件
     * @return
     */
    @Override
    public Wrapper condition() {
        QueryWrapper<QuestionBankEntity> query = new QueryWrapper<>();
        //维保项名称
        if (StringUtils.isNotBlank(opt)) {
            query.like("item", opt);
        }
        //驱动类型
        if (StringUtils.isNotBlank(driveType)) {
            query.eq("lift_category", driveType);
        }
        //维保周期
        if (StringUtils.isNotBlank(validTime)) {
            query.eq("type", validTime);
        }
        //启用状态
        if (null != statuz) {
            query.eq("status", statuz);
        }
        //按创建时间倒序
        query.orderByDesc("create_time");
        query.eq("1",1);

        return query;
    }

    /**
     * 分页条件
     * @return
     */
    @Override
    public IPage page() {
        Page<MaintenanceOptionEntity> page = new Page<>(getPageNum(), getPageSize());
        return page;
    }

}

