package cn.com.ty.lift.ud.templateBase.controller.query;

import cn.com.ty.lift.ud.common.BasePageQuery;
import cn.com.ty.lift.ud.question.mapper.entity.QuestionBankEntity;
import cn.com.ty.lift.ud.templateBase.mapper.entity.MaintenanceSetEntity;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

import java.util.Date;

/**
 * 维保套餐 查询条件
 *
 * @author Xson
 * @date 2020-04-29
 */
@Data
@ApiModel(description = "维保套餐 查询条件")
public final class MaintenanceSetQuery extends BasePageQuery {

    /**
     * 主键
     */
    @ApiModelProperty("主键")
    private Long id;

    /**
     * 分类 1:系统模板2:企业模板
     */
    @ApiModelProperty("分类 1:系统模板2:企业模板")
    private Integer type;

    /**
     * 公司id 所属企业
     */
    @ApiModelProperty("公司id 所属企业")
    private Integer companyId;

    /**
     * 名称 模板套餐名称
     */
    @ApiModelProperty("名称 模板套餐名称")
    private String name;

    /**
     * 总量 该模板套餐一共有多少个维保项类目
     */
    @ApiModelProperty("总量 该模板套餐一共有多少个维保项类目")
    private Integer amount;

    /**
     * 电梯类别(1:曳引梯;2:液压梯;3:杂物梯;4:自动扶梯;5:自动人行道)
     */
    @ApiModelProperty("电梯类别(1:曳引梯;2:液压梯;3:杂物梯;4:自动扶梯;5:自动人行道)")
    private String driveType;

    /**
     * 保养时间类型  ''1.半月，2.季度，3.半年，4.全年
     */
    @ApiModelProperty("保养时间类型  ''1.半月，2.季度，3.半年，4.全年")
    private String validTime;

    /**
     * 创建人
     */
    @ApiModelProperty("创建人")
    private String createBy;

    /**
     * 创建时间
     */
    @ApiModelProperty("创建时间")
    private Date createTime;

    /**
     * 更新人
     */
    @ApiModelProperty("更新人")
    private String updateBy;

    /**
     * 更新时间
     */
    @ApiModelProperty("更新时间")
    private Date updateTime;

    /**
     * 状态 0:上架1:下架
     */
    @ApiModelProperty("状态 0:上架1:下架")
    private Integer statuz;


    /**
     * 查询条件
     *
     * @return
     */
    @Override
    public Wrapper condition() {
        QueryWrapper<QuestionBankEntity> query = new QueryWrapper<>();
        //驱动类型
        if (StringUtils.isNotBlank(driveType)) {
            query.eq("drive_type", driveType);
        }
        //维保周期
        if (StringUtils.isNotBlank(validTime)) {
            query.eq("valid_time", validTime);
        }
        //启用状态
        if (null != statuz) {
            query.eq("statuz", statuz);
        }
        //模板套餐名称
        if (StringUtils.isNotBlank(name)) {
            query.like("name", name);
        }
        //分类
        if (null != type) {
            query.eq("type", type);
        }
        query.orderByDesc("create_time");
        query.eq("1",1);
        return query;
    }

    /**
     * 分页条件
     *
     * @return
     */
    @Override
    public IPage page() {
        Page<MaintenanceSetEntity> page = new Page<>(getPageNum(), getPageSize());
        return page;
    }

}

