package cn.com.ty.lift.ud.userOrders.service.impl;

import cn.com.ty.lift.ud.chat.mapper.ChatSessionMapper;
import cn.com.ty.lift.ud.chat.mapper.LiftCaseMapper;
import cn.com.ty.lift.ud.chat.mapper.entity.ChatSessionEntity;
import cn.com.ty.lift.ud.chat.mapper.entity.LiftCaseEntity;
import cn.com.ty.lift.ud.coupon.mapper.CouponMapper;
import cn.com.ty.lift.ud.coupon.service.ICouponService;
import cn.com.ty.lift.ud.dataBank.mapper.LiftBrandMapper;
import cn.com.ty.lift.ud.liftCaseAppeal.mapper.LiftCaseAppealMapper;
import cn.com.ty.lift.ud.liftCaseAppeal.service.ILiftCaseAppealService;
import cn.com.ty.lift.ud.operation.mapper.CommissionSettingMapper;
import cn.com.ty.lift.ud.operation.mapper.OtherSettingMapper;
import cn.com.ty.lift.ud.operation.service.ICommissionSettingService;
import cn.com.ty.lift.ud.payment.mapper.UserBillMapper;
import cn.com.ty.lift.ud.payment.mapper.entity.UserBillEntity;
import cn.com.ty.lift.ud.question.service.IQuestionBankService;
import cn.com.ty.lift.ud.redis.RedisUtils;
import cn.com.ty.lift.ud.userAccount.mapper.UserAccountMapper;
import cn.com.ty.lift.ud.userAccount.service.IUserAccountService;
import cn.com.ty.lift.ud.userInfo.mapper.UserInfoMapper;
import cn.com.ty.lift.ud.userInfo.mapper.entity.UserInfoEntity;
import cn.com.ty.lift.ud.userInfo.service.IUserInfoService;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jms.core.JmsMessagingTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;

/**
 * @author IyesKing
 * @version 1.0
 * @date 2020/12/23 10:47
 * 通过@Async注解表明该方法是一个异步方法， 如果注解在类级别上，则表明该类所有的方法都是异步方法，而这里的方法自动被注入使用ThreadPoolTaskExecutor作为TaskExecutor
 */

@Service("customMultiThreadingService")
public class CustomMultiThreadingService {


    private Logger logger = LoggerFactory.getLogger(CustomMultiThreadingService.class);

    @Resource
    private LiftCaseMapper caseMapper;
    @Resource
    private UserInfoMapper userInfoMapper;
    @Resource
    private CouponMapper couponMapper;
    @Resource
    private UserBillMapper userBillMapper;
    @Resource
    private ChatSessionMapper chatSessionMapper;


    /**
     * 取消问诊、出诊订单
     *
     * @param id
     */
    @Async
    public void liftCaseAysncTask(Long id) {

        LiftCaseEntity selectById = caseMapper.selectById(id);
        if (selectById.getAcceptStatus() == 9) {
            logger.info("订单已取消");
        } else {
            if (selectById.getDataTable() == 1 && selectById.getStatuz() == 1) {
                // 退款
                userInfoMapper.rechargePayBalance(selectById.getCreateUserId(), selectById.getPayCost() == null ? BigDecimal.ZERO : selectById.getPayCost());
                // 插入个人流水记录
                UserInfoEntity byUserId = userInfoMapper.getByUserId(selectById.getCreateUserId());
                UserBillEntity ube = new UserBillEntity();
                ube.setUserId(selectById.getCreateUserId());
                ube.setTargetUserId(selectById.getChargerId());
                ube.setBalance(byUserId.getBalance());
                ube.setType(2);
                ube.setBusinessType(14);
                ube.setDescr("取消订单退款到余额");
                ube.setFlowNum(selectById.getOrderSerialNumber());
                ube.setAmount(selectById.getPayCost() == null ? BigDecimal.ZERO : selectById.getPayCost());
                userBillMapper.insert(ube);
            }
            // 清除房间
            chatSessionMapper.delete(new QueryWrapper<ChatSessionEntity>()
                            .eq("data_id", id)
                            .eq("create_user_id", selectById.getCreateUserId())
//                .last("and FIND_IN_SET("+selectById.getCreateUserId()+",user_id)")
            );
            // 返还卡券
            if (selectById.getCouponId() != null) {
                couponMapper.updateUserCouponStatuz(selectById.getCouponId());
            }
            if (selectById.getRedEnvelopeId() != null) {
                couponMapper.updateUserCouponStatuz(selectById.getRedEnvelopeId());
            }
            // 修改诊单状态
            caseMapper.cancelCase(id);
        }

    }


}
